#ifndef BOOM_H_INCLUDED
#define BOOM_H_INCLUDED

/** Компилируем для GP2X или для Большого Брата. */
#define GP2X

/** Включение всех проверок времени исполнения. */
#define PARANOIC (true)
//#define PARANOIC (false)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>

#include "SDL/SDL.h"
#include "SDL/SDL_ttf.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_rotozoom.h"
#include "SDL/SDL_mixer.h"

namespace boom2x
{

/** Версия библиотеки. */
extern const char BOOM2X_VERSION[];

/** Восьмибитное беззнаковое целое. */
typedef unsigned char uint8;

/** Ширина дисплея GP2X - 320. */
extern const int WINDOW_WIDTH;
/** Высота дисплея GP2X - 240. */
extern const int WINDOW_HEIGHT;
/** Заголовок окна по умолчанию. */
extern const char WINDOW_TITLE[];


/** Кнопки джойстика gp2x. */
enum JoyButton
{
   jbUp              =0,
   jbUpLeft          =1,
   jbLeft            =2,
   jbDownLeft        =3,
   jbDown            =4,
   jbDownRight       =5,
   jbRight           =6,
   jbUpRight         =7,
   jbHome            =8,
   jbSelect          =9,
   jbShoulderLeft    =10,
   jbShoulderRight   =11,
   jbA               =12,
   jbB               =13,
   jbX               =14,
   jbY               =15,
   jbVolumeUp        =16,
   jbVolumeDown      =17,
   jbClick           =18
}; // enum JoyButton.


/** Системные события. */
enum SystemEvent
{
   seNone            =0,                  // Нет событий.
   seButton          =1,                  // Нажатие кнопки [джойстика].
   seStylusDown      =2,                  // Стилус опущен.
   seStylusUp        =3                   // Стилус поднят.
}; // enum SystemEvent


/** Логическое описание события. */
struct Event
{
   /** Тип события. */
   SystemEvent type;
   /** Нажатие кнопки. */
   JoyButton   button;
   /** Координата нажатия стилусом. */
   int         x;
   /** Координата нажатия стилусом. */
   int         y;
}; // struct Event


/** Базовый модуль.
   Каждый экран программы описывается одним модулем.
 */
class Module
{
protected:
   /** Модуль исполняется. */
   bool           executing;
   /** Остановить модуль. */
   void           stopExecution();

public:
   Module();
   virtual ~Module();

   /** Начальная инициализация, загрузка ресурсов. */
   virtual bool   setup    ()=0;
   /** Отрисовка экрана. */
   virtual void   draw     ()=0;

   /** Основная исполняющая функция. */
   virtual void   execute  ();

   /** Функция, выполняемая в момент простоя системы: для анимаций, ИИ и прочего. */
   virtual void   onIdle         ()=0;
   /** Реакция на нажатие клавиши [джойстика]. */
   virtual void   onButtonDown   (Event event)=0;
   /** Реакция на нажатие стилусом [мышью]. */
   virtual void   onStylusDown   (Event event)=0;
   /** Реакция на поднятие стилуса [мыши]. */
   virtual void   onStylusUp     (Event event)=0;
}; // class Module


/** Базовая функциональность программы для GP2X. */
class Toolkit
{
public:
   /** Основной каталог ресурсов. */
   static const char resources[];

public:
   /** Спрайт поверхности экрана. */
   SDL_Surface*   screen;
   /** Доступ к джойстику. Должен быть подключён для отлова кнопок gp2x. */
   SDL_Joystick*  joystick;
   /** Шрифт по умолчанию. */
   TTF_Font*      font;
   /** Текущий активный цвет. */
	SDL_Color      foreColour;

public:
   Toolkit();
   ~Toolkit();

   /** Инициализировать приложение. */
   bool  start       ();
   /** Остановить приложение, освободить захваченные ресурсы. */
   void  shutdown    ();

   /** Выбрать следующее сообщение из очереди. */
   bool  getEvent          (Event* event);
   /** Очистить очередь сообщений. */
   void  clearEvents       ();

   /** Загрузить шрифт заданной гарнитуры кегля size. */
   TTF_Font*      loadFont (const char* filename, uint8 size);
   /** Выгрузить шрифт. */
   void           freeFont (TTF_Font* aFont);
   /** Установить шрифт. */
   void           setFont  (TTF_Font* aFont);

   /** Установить активный цвет. */
   void  setForeColour  (int aR, int aG, int aB);

   /** Вывести на поверхность строку заданным цветом. */
   void  drawString     (SDL_Surface* surface, const char* value, int x, int y, SDL_Colour colour);
   /** Вывести на поверхность строку. */
   void  drawString     (SDL_Surface* surface, const char* value, int x, int y);
   /** Вывести на поверхность целое заданным цветом. */
   void  drawInt        (SDL_Surface* surface, int value, int x, int y, SDL_Colour colour);
   /** Вывести на поверхность целое. */
   void  drawInt        (SDL_Surface* surface, int value, int x, int y);
   /** Вывести на поверхность спрайт. */
   void drawSprite      (SDL_Surface* surface, SDL_Surface* sprite, int srcX, int srcY, int dstX, int dstY, int width, int height);

}; // class System

/** Глобальный экземпляр для простого доступа. */
extern Toolkit kit;

} // namespace boom2x

#endif // BOOM_H_INCLUDED
