//#ifndef GP2X
//#define _CRT_SECURE_NO_DEPRECATE
//#endif

#include "boom.h"

using namespace boom2x;

//------------------------------------------------------------------------------

/** Модуль "О программе". */
class ModuleAbout: public Module
{
private:
   /** Рисунок фона. */
   SDL_Surface*   background;

public:
   ModuleAbout();
   ~ModuleAbout();

   /** Начальная инициализация, загрузка ресурсов. */
   bool  setup    ();
   /** Отрисовка экрана. */
   void  draw     ();

   /** Основная исполняющая функция. */
   void  execute  ();

   /** Функция, выполняемая в момент простоя системы: для анимаций, ИИ и прочего. */
   void  onIdle         ();
   /** Реакция на нажатие клавиши [джойстика]. */
   void  onButtonDown   (Event event);
   /** Реакция на нажатие стилусом [мышью]. */
   void  onStylusDown   (Event event);
   /** Реакция на поднятие стилуса [мыши]. */
   void  onStylusUp     (Event event);
}; // class ModuleAbout


ModuleAbout::ModuleAbout()
: Module()
{
   background=NULL;
}

ModuleAbout::~ModuleAbout()
{
   SDL_FreeSurface(background);
}

bool ModuleAbout::setup()
{
   // Если рисунок фона уже загружен, ничего делать не надо - модуль уже настроен.
   if(NULL!=background)
      return true;

	// Рисунок фона.
	SDL_Surface* img_temp=IMG_Load("res/about/back.bmp");
	if(!img_temp)
	{
		printf("boom2x: ModuleAbout::setup(): No background file.\n");
		return false;
	}

	// Преобразовать в понятный железу формат.
	background=SDL_ConvertSurface(img_temp, kit.screen->format, SDL_HWSURFACE);
   SDL_FreeSurface(img_temp);

#if PARANOIC
	if (!background)
	{
      printf("boom2x: ModuleAbout::setup(): couldn't create background.\n");
      return false;
	}
#endif

   return true;
}

// Основная исполняющая функция.
void ModuleAbout::execute()
{
   Module::execute();
}

// Функция, выполняемая в момент простоя системы: для анимаций, ИИ и прочего.
void ModuleAbout::onIdle()
{
   //
}

void ModuleAbout::onButtonDown(Event event)
{
   stopExecution();
}

void ModuleAbout::onStylusDown(Event event)
{
   stopExecution();
}

void ModuleAbout::onStylusUp(Event event)
{
   //
}

void ModuleAbout::draw()
{
	kit.drawSprite(kit.screen, background, 0, 0, 0, 0, background->w, background->h);
	SDL_Flip(kit.screen);
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------

/** Основной модуль игры - поле с фишками. */
class ModuleGame: public Module
{
private:
   /** Рисунок фона. */
   SDL_Surface*   background;

   /** Количество фигур. */
   static const int PIECES=15;
   /** Изображения фигур. */
   SDL_Surface*   pieces[PIECES];

   /** Размер игрового поля. */
   static const int SIZE=4;
   /** Игровое поле. */
   int            field[SIZE][SIZE];
   /** Количество действий, предпринятых игроком. */
   int            actions;
   /** Каким цветом отображать количество действий. */
   bool           hilite_actions;
   /** Фишки перепутаны. */
   bool           messed;
   /** Головоломка решена. */
   bool           solved();

private:
   /** Загрузить одну фишку по имени. */
   SDL_Surface*   loadPiece(const char* filename);
   /** Шрифт для вывода количества сделанных ходов. */
   TTF_Font*      font;

   /** Привести поле в исходное (правильное) состояние. */
   void  fieldReset     ();
   /** Перемешать фишки. */
   void  fieldMess      ();
   /** Найти координаты пустой клетки. */
   void  findNull       (int& x, int& y);
   /** Передвинуть фишку вверх. */
   bool  moveUp         ();
   /** Передвинуть фишку вниз. */
   bool  moveDown       ();
   /** Передвинуть фишку влево. */
   bool  moveLeft       ();
   /** Передвинуть фишку вправо. */
   bool  moveRight      ();

   /** Вызов модуля "О программе" */
   void  about          ();

public:
   ModuleGame();
   ~ModuleGame();

   /** Начальная инициализация, загрузка ресурсов. */
   bool  setup    ();
   /** Отрисовка экрана. */
   void  draw     ();

   /** Основная исполняющая функция. */
   void  execute  ();

   /** Функция, выполняемая в момент простоя системы: для анимаций, ИИ и прочего. */
   void  onIdle         ();
   /** Реакция на нажатие клавиши [джойстика]. */
   void  onButtonDown   (Event event);
   /** Реакция на нажатие стилусом [мышью]. */
   void  onStylusDown   (Event event);
   /** Реакция на поднятие стилуса [мыши]. */
   void  onStylusUp     (Event event);

}; // class ModuleGame


ModuleGame::ModuleGame()
: Module()
{
   background=NULL;
   fieldReset();
   hilite_actions=false;

   srand( time(NULL) );                            // Инициализировать генератор случайных чисел.
}

ModuleGame::~ModuleGame()
{
   kit.freeFont(font);
   SDL_FreeSurface(background);
   for(int i=0; i<PIECES; ++i)
   {
      SDL_FreeSurface(pieces[i]);
   }
}

bool ModuleGame::setup()
{
   // Если рисунок фона уже загружен, ничего делать не надо - модуль уже настроен.
   if(NULL!=background)
      return true;

	// Рисунок фона.
	SDL_Surface* img_temp=IMG_Load("res/back.bmp");
	if(!img_temp)
	{
		printf("boom2x: ModuleGame::setup(): No background file.\n");
		return false;
	}

	// Преобразовать в понятный железу формат.
	background=SDL_ConvertSurface(img_temp, kit.screen->format, SDL_HWSURFACE);
   SDL_FreeSurface(img_temp);

#if PARANOIC
	if (!background)
	{
      printf("boom2x: ModuleGame::setup(): couldn't create background.\n");
      return false;
	}
#endif

   // Загрузим фишки.
   pieces[ 0]=loadPiece("res/01.bmp");
   pieces[ 1]=loadPiece("res/02.bmp");
   pieces[ 2]=loadPiece("res/03.bmp");
   pieces[ 3]=loadPiece("res/04.bmp");
   pieces[ 4]=loadPiece("res/05.bmp");
   pieces[ 5]=loadPiece("res/06.bmp");
   pieces[ 6]=loadPiece("res/07.bmp");
   pieces[ 7]=loadPiece("res/08.bmp");
   pieces[ 8]=loadPiece("res/09.bmp");
   pieces[ 9]=loadPiece("res/10.bmp");
   pieces[10]=loadPiece("res/11.bmp");
   pieces[11]=loadPiece("res/12.bmp");
   pieces[12]=loadPiece("res/13.bmp");
   pieces[13]=loadPiece("res/14.bmp");
   pieces[14]=loadPiece("res/15.bmp");

   // Шрифт количества ходов.
   font=kit.loadFont("res/crystal.ttf", 22);
   kit.setFont(font);

   return true;
}

// Загрузить одну фишку по имени.
SDL_Surface* ModuleGame::loadPiece(const char* filename)
{
   SDL_Surface* img_piece=IMG_Load(filename);
#if PARANOIC
   if(!img_piece)
   {
      printf("boom2x: ModuleGame::loadPiece(): No piece file (%s).\n", filename);
      return NULL;
   }
#endif
   SDL_Surface* piece=SDL_ConvertSurface(img_piece, kit.screen->format, SDL_HWSURFACE);
   SDL_FreeSurface(img_piece);
   return piece;
}


// Привести поле в исходное (правильное) состояние.
void ModuleGame::fieldReset()
{
   for(int i=0; i<SIZE; i++)
   {
      for(int j=0; j<SIZE; j++)
      {
         field[i][j]=i*SIZE+j+1;
      }
   }
   field[SIZE-1][SIZE-1]=0;
   messed=false;
   actions=0;
}

// Перемешать фишки.
void ModuleGame::fieldMess()
{
   // Перемешивание медленное и не обязательно сильно путает, но гарантирует возможность собрать головоломку.
   int moves=250;
   while(moves)
   {
      int r=rand()%4;
      switch(r)
      {
      case  0: if(moveDown()) moves--; break;
      case  1: if(moveUp()) moves--; break;
      case  2: if(moveLeft()) moves--; break;
      case  3: if(moveRight()) moves--; break;
      }
   }
   messed=true;
   actions=0;
}

// Найти координаты пустой клетки.
void ModuleGame::findNull(int& x, int& y)
{
   for(int i=0; i<SIZE; i++)
   {
      for(int j=0; j<SIZE; j++)
      {
         if( 0==field[i][j] )
         {
            x=j;
            y=i;
            return;
         }
      }
   }
}

// Передвинуть фишку вверх.
bool ModuleGame::moveUp()
{
   int x, y;
   findNull(x, y);
   if(y < SIZE-1)
   {
      field[y][x]=field[y+1][x];
      field[y+1][x]=0;
      actions++;
      return true;
   }
   return false;
}

// Передвинуть фишку вниз.
bool ModuleGame::moveDown()
{
   int x, y;
   findNull(x, y);
   if(y > 0)
   {
      field[y][x]=field[y-1][x];
      field[y-1][x]=0;
      actions++;
      return true;
   }
   return false;
}

// Передвинуть фишку влево.
bool ModuleGame::moveLeft()
{
   int x, y;
   findNull(x, y);
   if(x < SIZE-1)
   {
      field[y][x]=field[y][x+1];
      field[y][x+1]=0;
      actions++;
      return true;
   }
   return false;
}

// Передвинуть фишку вправо.
bool ModuleGame::moveRight()
{
   int x, y;
   findNull(x, y);
   if(x >0)
   {
      field[y][x]=field[y][x-1];
      field[y][x-1]=0;
      actions++;
      return true;
   }
   return false;
}

// Головоломка решена.
bool ModuleGame::solved()
{
   for(int i=0; i<SIZE; i++)
   {
      for(int j=0; j<SIZE; j++)
      {
         int cell=i*SIZE+j;
         if( (field[i][j] != cell+1) && cell<15)
            return false;
      }
   }
   return true;
}


// Вызов модуля "О программе".
void ModuleGame::about()
{
   ModuleAbout ma;
   ma.setup();
   ma.execute();
}


// Основная исполняющая функция.
void ModuleGame::execute()
{
   Module::execute();
}

// Функция, выполняемая в момент простоя системы: для анимаций, ИИ и прочего.
void ModuleGame::onIdle()
{
   //
}

void ModuleGame::onButtonDown(Event event)
{
   switch(event.button)
   {
   case  jbUp     :
         moveUp();
         break;
   case  jbDown   :
         moveDown();
         break;
   case  jbLeft   :
         moveLeft();
         break;
   case  jbRight  :
         moveRight();
         break;

   case  jbA:
         fieldMess();
         break;

   case  jbB:
         fieldReset();
         break;

   case  jbY:
         about();
         break;

   case  jbHome:
         stopExecution();
         break;

   default:
         break;
   }

   draw();

   // Собрана ли головоломка?
   if(executing && messed && solved())
   {
      messed=false;
      for(int i=0; i<4; i++)
      {
         hilite_actions=true;
         draw();
         SDL_Delay(100);
         hilite_actions=false;
         draw();
         SDL_Delay(100);
      }
      actions=0;
      draw();
   }
}

void ModuleGame::onStylusDown(Event event)
{
   //
}

void ModuleGame::onStylusUp(Event event)
{
   //
}

void ModuleGame::draw()
{
   // Фон.
	kit.drawSprite(kit.screen, background, 0, 0, 0, 0, background->w, background->h);

   // Фишки.
   for(int i=0; i<SIZE; i++)
   {
      int y = i*58 + 4;
      for(int j=0; j<SIZE; j++)
      {
         int x = j*58 + 4;
         int piece=field[i][j];
         if(0!=piece)
            kit.drawSprite(kit.screen, pieces[piece-1], 0, 0, x+2, y+2, pieces[0]->w, pieces[0]->h);
      }
   }

   // Количество ходов.
   const int LEDS=6;
   char s[LEDS+1] = { '\0' };
	sprintf(s, "%06d", actions);

   hilite_actions ? kit.setForeColour(255, 255, 255) : kit.setForeColour(0, 255, 0);
   kit.drawString(kit.screen, s, 249, 8);

   // Сбросить буфер на экран.
	SDL_Flip(kit.screen);
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------

/** Заголовочный модуль игры - аналогичен модулю "О программе" */
class ModuleTitle: public ModuleAbout
{
private:
   /** Вызов основного игрового модуля. */
   void  game  ();

public:
   /** Реакция на нажатие клавиши [джойстика]. */
   void  onButtonDown   (Event event);
   /** Реакция на нажатие стилусом [мышью]. */
   void  onStylusDown   (Event event);
}; // class ModuleTitle


// Вызов основного игрового модуля.
void ModuleTitle::game()
{
   ModuleGame mg;
   mg.setup();
   mg.execute();
   stopExecution();
}

void ModuleTitle::onButtonDown(Event event)
{
   game();
}

void ModuleTitle::onStylusDown(Event event)
{
   game();
}
//------------------------------------------------------------------------------


int main(int argc, char **argv)
{
   kit.start();
   ModuleTitle mt;
   mt.setup();
   mt.execute();
	kit.shutdown();
	return 0;
}
